

import java.util.Iterator;

public class TreeNode<T> {

	private TreeNode<T> left; 
	private TreeNode<T> right; 
	private TreeNode<T> parent; 
	private T value; 
	
	public TreeNode(T value) {
		this.value = value; 
	}
	
	//getters
	public TreeNode<T> getRight() {
		return right; 
	}
	
	public TreeNode<T> getLeft() {
		return left; 
	}
	
	public TreeNode<T> getParent() {
		return parent; 
	}
	
	public T getValue() {
		return value; 
	}
	
	public int getHeight() {
		if(isLeaf()) return 0; 
		
		return 1 + Math.max(left.getHeight(),right.getHeight()); 
	}
	
	public int getDepth() {
		if(isRoot()) return 0; 
		
		return 1 + parent.getDepth(); 
	}
	
	public int getSize() {
		if(isLeaf()) return 1; 
		
		return 1 + left.getSize() + right.getSize(); 
	}
	
	//setters
	public void setLeft(TreeNode<T> left) {
		this.left = left; 
		if(left != null) left.parent = this; // in case you want to set left to null to detach from tree 
	}
	
	public void setRight(TreeNode<T> right) {
		this.right = right; 
		if(right != null) right.parent = this; 
		
	}
	
	public void setValue(T val) {
		value = val; 
	}
	
	//helper methods
	public boolean hasLeft() {
		return left != null; 
	}
	
	public boolean hasRight() {
		return right != null; 
	}
	
	public boolean hasParent() {
		return parent != null; 
	}
	
	public boolean isRoot() {
		return parent == null; 
	}
	
	public boolean isLeaf() {
		return left == null && right == null; 
	}
	
	public boolean isInternal() {
		return !isLeaf(); 
	}
	
	public TreeNode<T> least() {
		if(hasLeft()) return left.least(); 
		return this; 
	}
	
	public TreeNode<T> greatest() {
		if(hasRight()) return right.greatest(); 
		return this; 
	}
	
	public String inOrder() {
		String tree = ""; 
		
		if(hasLeft())tree += left.inOrder(); 
		tree += this.getValue() + " ";
		if(hasRight()) tree += right.inOrder(); 
		
		return tree; 
	}
	
	public String preOrder() {
		String tree = ""; 
		
		tree += this.getValue() + " ";
		if(hasLeft())tree += left.preOrder(); 
		if(hasRight()) tree += right.preOrder(); 
		
		return tree; 
		
	}
	
	public String postOrder() {
		String tree = ""; 
		
		if(hasLeft())tree += left.postOrder(); 
		if(hasRight()) tree += right.postOrder(); 
		tree += this.getValue() + " ";
		
		return tree; 
		
	}
	
	protected TreeNode<T> successor(TreeNode<T> node) {
		if(node.hasRight()) return node.getRight().least(); 
		
		TreeNode<T> p = node.getParent(); 
		
		while(p != null && node == p.getRight()) {
			node = p; 
			p = p.getParent(); 
		}
		
		return p; 
		
	}
	
	protected TreeNode<T> predecessor(TreeNode<T> node) {
		if(node.hasLeft()) return node.getLeft().greatest(); 
		
		TreeNode<T> p = node.getParent(); 
		
		while(p != null && node == p.getLeft()) {
			node = p; 
			p = p.getParent(); 
		}
		
		return p;
	}
		
	
	public String toString() {
		return inOrder(); 
	}
	
	public void remove() { 
		if(isLeaf()) {
			if(parent.left== this) parent.setLeft(null);
			else parent.setRight(null);
		} else if(left == null) {
			if(parent.left == this) parent.setLeft(right);
			else parent.setRight(right);
		} else if(right == null) {
			if(parent.right == this) parent.setRight(left);
			else parent.setLeft(left);
		} else {
			TreeNode<T> succ = successor(this); 
			this.value = succ.value; 
			succ.remove(); 
		}
	}
	



}
