/*
 * Tara Graeve
 * Project 5: Windows 
 * 6/3/19
 */


import java.awt.Color;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import edu.princeton.cs.introcs.*;

public class WindowDisplay implements DrawListener {
	private ArrayList<Window> windows; 
	private Draw d;
	private ArrayList<Window> origWindows; // needed to keep track of state of windows before they are moved

	public WindowDisplay(int xScale, int yScale) {
		d = new Draw(); 
		d.addListener(this);

		windows = new ArrayList<>(); 
		origWindows = new ArrayList<>(); 


		d.setCanvasSize(xScale, yScale);
		d.setXscale(0,xScale); 
		d.setYscale(0,yScale); 
	}

	public void addWindow(Window w) {
		windows.add(w); // add window to both main ArrayList and "original" ArrayList that's keeping track of the original state of the array (later needed to reset)
		
		origWindows.add(new Window(w)); 
	}

	public void draw() {
		for (Window i: windows) {
			i.draw();  
		}
	}

	public void reset() {
		d.clear(); // clear screen 
		windows = new ArrayList<>(); // reset windows to empty array list 
		for (Window i: origWindows) { // redraw original state
			windows.add(new Window(i)); 
			i.draw(); 
		}

	}



	//window inner class
	public class Window {
		private double centerX; 
		private double centerY; 
		private double width; 
		private double height; 
		private Color color; 

		public Window(double x, double y, double w, double h, Color c) {
			centerX = x; 
			centerY = y; 
			width = w; 
			height = h; 
			color = c; 
		}

		public Window(Window w) { // added this constructor: needed to properly add and update the ArrayLists while maintaining proper functionality 
			this.centerX = w.centerX; 
			this.centerY = w.centerY; 
			this.width = w.width; 
			this.height = w.height; 
			this.color = w.color; 
		}

		public void draw() {
			d.setPenColor(color); 
			d.filledRectangle(centerX,centerY,width/2,height/2); 
		} 

		public boolean isClicked(double x, double y) {
			if((x  <= centerX + width/2 && x >= centerX - width/2) && (y <= centerY + height/2 && y >= centerY - height/2)) {
				return true;
			} else return false; 
		}


	}
	//end window inner class



	@Override
	public void mousePressed(double x, double y) {
		for(int i = 0; i < windows.size(); i++) {
			if(windows.get(i).isClicked(d.mouseX(),d.mouseY())) {
				windows.add(windows.get(i)); // move to top
				windows.remove(windows.get(i)); // delete old window 
				windows.get(i).draw(); // redraw
			}
		}

		for(Window i: windows) {
			i.draw(); // draw all windows
		}

	}

	@Override
	public void mouseDragged(double x, double y) {
		Window win = windows.get(windows.size()-1); // tracks the top window

		if (win.isClicked(d.mouseX(),d.mouseY())) { // checks if top window is clicked and updates accordingly when dragged 
			win.centerX = d.mouseX(); 
			win.centerY = d.mouseY(); 
			d.clear(); // prevents "trail" of window as it is dragged 
			win.draw(); // draw updated window

			draw(); // draws the rest of the windows so they don't disappear when you drag another window 

		}
	}

	@Override
	public void mouseReleased(double x, double y) {
		// TODO Auto-generated method stub

	}

	@Override
	public void keyTyped(char c) {


	}

	@Override
	public void keyPressed(int keycode) {
		if (keycode == KeyEvent.VK_Q) {
			System.exit(1);
		} else if (keycode == KeyEvent.VK_R) {
			reset(); 
		}

	}

	@Override
	public void keyReleased(int keycode) {
		// TODO Auto-generated method stub

	}

}
