/* 
 Tara Graeve 
 COMP 1672 Project 1: Connect 4 
 GameBoard class for Project 1 
 4/15/19
 */

package Connect4;

import edu.princeton.cs.introcs.StdDraw;

public class GameBoard {
	private Cell[][] board; // array for game board 

	private int width = 700; // added instance variables for width and height 
	private int height = 600; 
	private int row; 
	
	public GameBoard() { 
		board = new Cell[7][6]; 

		StdDraw.setCanvasSize(width,height); // setting up the StdDraw canvas 
		StdDraw.setXscale(0,width); 
		StdDraw.setYscale(0,height); 
		StdDraw.setPenColor(StdDraw.DARK_GRAY); 
		StdDraw.filledRectangle(width/2, height/2, width/2, height/2); // fill entire canvas with black to serve as the "frame" of the game board

		for (int i = 0; i < board.length; i++) {         // draw game board with "holes" for checkers to go in 
			for (int j = 0; j < board[i].length; j++) {
				board[i][j] = new Cell(Cell.State.Empty); // create an initially empty cell in each location of game board
			}
		}

	}

	public GameBoard(int height) { // does the same thing as the no argument constructor, but allows the height to be changed 
		board = new Cell[7][6]; 

		StdDraw.setCanvasSize(width,height); 
		StdDraw.setXscale(0,width); 
		StdDraw.setYscale(0,height); 

		StdDraw.filledRectangle(width/2, height/2, width/2, height/2);


		for (int i = 0; i < board.length; i++) {
			for (int j = 0; j < board[i].length; j++) {
				board[i][j] = new Cell(Cell.State.Empty);
			}
		}
	}


	public boolean columnFull(int colNum) { 
		for (int i = 0; i < board.length-1; i++) {
			if(board[colNum][i].getState() == Cell.State.Empty) { // check to see if at given position cell is empty
				return false; // if a cell is empty then the column is not full, so false is returned
			}
		}
		return true; // if false has not been returned then the column is full, so true is returned
	}


	public boolean addChecker(int colNum, Cell.State s) {
		if (colNum < 0 || colNum > board.length) { // check to make sure column is within the board (array bounds) -- can't place a checker in a non-existent place 
			return false; 
		}
		if (columnFull(colNum) == true) { // check to see if column is full -- if it's full, it can't successfully be placed
			return false; 
		}
		
		for(int i = 0; i < board[colNum].length; i++) { // if these conditions haven't been returned false, this loop will execute 
		
			if (board[colNum][i].getState() == Cell.State.Empty) { // check to see if the cell at the current position is empty 
				board[colNum][i].setState(s); // if it's empty then its state can be set 
				row = i; 
				break; // prevents loop from executing over and over again during the same turn until a column is full 
			}
			
		}
		
		return true; // checker has been successfully placed 

	}
	
	public int getRow() { // getter for row of last checker placed to be used in gameWon method 
		return row; 
	}

	public void drawGameBoard() { 
		for (int i = 0; i < board.length; i++) {
			for (int j = 0; j < board[i].length; j++) {
				if (board[i][j].getState() == Cell.State.Empty) { // if-else if-else statements to identify cell state and fill cell in with corresponding checker 
					StdDraw.setPenColor(StdDraw.WHITE); 
				} 
				else if (board[i][j].getState() == Cell.State.Red) {
					StdDraw.setPenColor(StdDraw.BOOK_RED); 
				} 
				else {
					StdDraw.setPenColor(StdDraw.BOOK_BLUE);
				}
				StdDraw.filledCircle(i*(width/7)+((width/7)/2),j*(height/6)+((height/6)/2),45); // draws game board but modifies to fill screen based on value of height argument
			}
		}
	} 

	public boolean boardFull() { 
		for(int i = 0; i < board.length; i++) { 
			for (int j = 0; j < board[i].length; j++) { 
				if(board[i][j].getState() == Cell.State.Empty) { // check to see if at any location a cell is empty 
					return false; // if any cell is empty, the game board isn't full. Return false.
				}
			}
		}
		return true; 
	}

	public boolean gameWon(int row, int col) { 
		int count1 = 1; // tracks vertical number of checkers in a row 
		int count2 = 1; // tracks horizontal number of checkers in a row 
		
		
		// check vertical 
		for (int i = 0; i < board[col].length-2; i++) {
			if ((board[col][i].getState() == board[col][i+1].getState()) && board[col][i].getState() != Cell.State.Empty) {
				count1++; // if the condition is true then there is multiple in a row, so increment tracker variable 
			} else { 
				count1 = 1; // if they aren't the same then reset the counter 
			}
			if (count1 >= 4) {
				count1 = 1;
				return true; // this means there's 4 in a row! The game is won! 
			}
		}
	
		
		
		// check horizontal 
		for (int i = 0; i < board[col].length-2; i++) { 
			if ((board[i][row].getState() == board[i+1][row].getState()) && board[i][row].getState() != Cell.State.Empty) {
				count2++; 
			} else { 
				count2 = 1; 
			}
			if (count2 >= 4) {
				count2 = 1; 
				return true; 
			}
		}
		
		return false; 
	}

}
