--Written by: Tara Graeve
--Peer review by: Maddie Thompson
--Tara's Twisters Gymnastics database
--4/22/2020
 
 --create the database
 IF NOT EXISTS(SELECT * FROM sys.databases
	WHERE NAME = N'TarasTwistersGymnastics')
	CREATE DATABASE TarasTwistersGymnastics
 GO 
 USE TarasTwistersGymnastics 
 GO 

 --location of csv files
 DECLARE @data_path NVARCHAR(256); 
 SELECT @data_path = 'C:\Users\mocha\OneDrive\Documents\DBTables3240\'; 

 --delete existing tables 
 IF EXISTS (
	SELECT * FROM sys.tables 
	WHERE NAME = N'LEVEL_COACH') 
	DROP TABLE LEVEL_COACH; 

 IF EXISTS (
	SELECT * FROM sys.tables 
	WHERE NAME = N'GYMNAST_LEVEL') 
	DROP TABLE GYMNAST_LEVEL; 

 IF EXISTS (
	SELECT * FROM sys.tables 
	WHERE NAME = N'SCORE') 
	DROP TABLE SCORE;
	
 IF EXISTS (
	SELECT * FROM sys.tables 
	WHERE NAME = N'PREV_EXPERIENCE') 
	DROP TABLE PREV_EXPERIENCE; 

 IF EXISTS (
	SELECT * FROM sys.tables 
	WHERE NAME = N'GYMNAST') 
	DROP TABLE GYMNAST;
	
 IF EXISTS (
	SELECT * FROM sys.tables 
	WHERE NAME = N'HOUSEHOLD') 
	DROP TABLE HOUSEHOLD; 

 IF EXISTS (
	SELECT * FROM sys.tables 
	WHERE NAME = N'COACH') 
	DROP TABLE COACH;
	
 IF EXISTS (
	SELECT * FROM sys.tables 
	WHERE NAME = N'EVENT') 
	DROP TABLE EVENT;
	
 IF EXISTS (
	SELECT * FROM sys.tables 
	WHERE NAME = N'LEVEL') 
	DROP TABLE LEVEL; 
 
 -- create tables 

 --level table 
 CREATE TABLE LEVEL 
 (Level_ID				NVARCHAR(15) CONSTRAINT pk_LevelID PRIMARY KEY, 
  Level_Number			INT CONSTRAINT ck_Level CHECK (Level_Number >= 3 AND Level_Number <= 10) NOT NULL, 
  Level_Requirements	NVARCHAR(150) NOT NULL); 
 
 --event table
 CREATE TABLE EVENT 
 (Event_ID			NCHAR(2) CONSTRAINT pk_EventID PRIMARY KEY, 
  Event_Name		NVARCHAR(15) NOT NULL, 
  Event_Description	NVARCHAR(200) NOT NULL); --check lengths of these fields that I typed in 
 
 --coach table
 CREATE TABLE COACH 
 (Coach_ID		INT IDENTITY(1,1) CONSTRAINT pk_CoachID PRIMARY KEY, 
  Coach_FName	NVARCHAR(20) NOT NULL, 
  Coach_LName	NVARCHAR(35) NOT NULL, 
  Coach_DoB		DATE NOT NULL, 
  Coach_Phone 	NCHAR(10) NOT NULL, 
  Coach_Email 	NVARCHAR(25) NOT NULL); 
 
 --household table 
 CREATE TABLE HOUSEHOLD
 (Household_ID				INT IDENTITY(1,1) CONSTRAINT pk_HouseholdID PRIMARY KEY, 
  Household_LName			NVARCHAR(35) NOT NULL, 
  Household_Email			NVARCHAR(35) NOT NULL, 
  Household_Phone			NCHAR(10) NOT NULL, 
  Household_Primary_Contact NVARCHAR(20) NOT NULL); 
  
 --gymnast table 
 CREATE TABLE GYMNAST 
 (Gymnast_ID	INT IDENTITY(1,1) CONSTRAINT pk_GymanstID PRIMARY KEY, 
  Household_ID	INT CONSTRAINT fk_HouseholdID FOREIGN KEY 
	REFERENCES HOUSEHOLD(Household_ID),
  Gymnast_FName	NVARCHAR(25) NOT NULL, 
  Gymnast_LName	NVARCHAR(35) NOT NULL, 
  Gymnast_DoB	DATE NOT NULL); 
  
 --prev experience table
 CREATE TABLE PREV_EXPERIENCE 
 (PrevExp_ID			INT IDENTITY(1,1) CONSTRAINT pk_PrevExpID PRIMARY KEY, 
  Gymnast_ID			INT CONSTRAINT fk_GymnastID FOREIGN KEY 
	REFERENCES GYMNAST(Gymnast_ID), 
  Gym_Name				NVARCHAR(50) NOT NULL, 
  Gym_City				NVARCHAR(22) NOT NULL,
  Gym_State				NCHAR(2) NOT NULL, 
  Gymnast_Start_Date 	DATE NOT NULL,
  Gymnast_End_Date		DATE NOT NULL, 
  Gymnast_Level_Reached	INT CONSTRAINT ck_LevelReached CHECK (Gymnast_Level_Reached >=3 AND Gymnast_Level_Reached <= 10) NOT NULL); 
  
 --score table
 CREATE TABLE SCORE 
 (Score_ID				INT IDENTITY(1,1) CONSTRAINT pk_ScoreID PRIMARY KEY, 
  Gymnast_ID			INT CONSTRAINT fk_GymnastID_Score FOREIGN KEY 
	REFERENCES GYMNAST(Gymnast_ID),
  Event_ID				NCHAR(2) CONSTRAINT fk_EventID FOREIGN KEY 
	REFERENCES EVENT(Event_ID), 
  Score					NUMERIC(5,3) CONSTRAINT ck_Score CHECK (Score >= 0 AND Score <= 10) NOT NULL, 
  Routine_Description	NVARCHAR(150) NOT NULL, 
  Routine_Feedback		NVARCHAR(150) NOT NULL,
  Score_Date			DATE NOT NULL);		
  
 --gymnast level table 
 CREATE TABLE GYMNAST_LEVEL
 (Gymnast_ID	INT NOT NULL, 
  Level_ID		NVARCHAR(15) NOT NULL, 
  Season_Number	INT NOT NULL, 
  Gymnast_Level	INT CONSTRAINT ck_GymnastLevel CHECK (Gymnast_Level >= 3 AND Gymnast_Level <= 10) NOT NULL,
  Season_Year	INT NOT NULL,
  CONSTRAINT pk_GymnastLevel PRIMARY KEY (Gymnast_ID,Level_ID,Season_Number),
  CONSTRAINT fk_LevelID FOREIGN KEY (Level_ID) REFERENCES LEVEL(Level_ID),
  CONSTRAINT fk_GymnastID_GymnastLevel FOREIGN KEY (Gymnast_ID) REFERENCES GYMNAST(Gymnast_ID)); 
  
  --level coach table
 CREATE TABLE LEVEL_COACH 
 (Coach_ID			INT NOT NULL, 
  Level_ID			NVARCHAR(15) NOT NULL, 
  Start_Date		DATE NOT NULL, 
  Level_Assigned	INT CONSTRAINT ck_LevelAssigned CHECK (Level_Assigned >= 3 AND Level_Assigned <= 10), 
  Coach_Duties		NVARCHAR(100) NOT NULL,
  CONSTRAINT pk_LevelCoach PRIMARY KEY (Coach_ID, Level_ID,Start_Date), 
  CONSTRAINT fk_CoachID FOREIGN KEY (Coach_ID) REFERENCES COACH(Coach_ID), 
  CONSTRAINT fk_LevelID_LevelCoach FOREIGN KEY (Level_ID) REFERENCES LEVEL(Level_ID)); 

  --load data into tables 

  --level
  EXECUTE ( N'BULK INSERT LEVEL FROM ''' + @data_path + N'LEVEL.csv''
  WITH (
	CHECK_CONSTRAINTS, 
	CODEPAGE=''ACP'', 
	DATAFILETYPE = ''char'',
	FIELDTERMINATOR=''\t'', 
	ROWTERMINATOR = ''\n'', 
	KEEPIDENTITY, 
	TABLOCK
	); 
	');

  --event
  EXECUTE ( N'BULK INSERT EVENT FROM ''' + @data_path + N'EVENT.csv''
  WITH (
	CHECK_CONSTRAINTS, 
	CODEPAGE=''ACP'', 
	DATAFILETYPE = ''char'',
	FIELDTERMINATOR='','', 
	ROWTERMINATOR = ''\n'', 
	KEEPIDENTITY, 
	TABLOCK
	); 
	');

  --coach 
  EXECUTE ( N'BULK INSERT COACH FROM ''' + @data_path + N'COACH.csv''
  WITH (
	CHECK_CONSTRAINTS, 
	CODEPAGE=''ACP'', 
	DATAFILETYPE = ''char'',
	FIELDTERMINATOR='','', 
	ROWTERMINATOR = ''\n'', 
	KEEPIDENTITY, 
	TABLOCK
	); 
	');

  --household
  EXECUTE ( N'BULK INSERT HOUSEHOLD FROM ''' + @data_path + N'HOUSEHOLD.csv''
  WITH (
	CHECK_CONSTRAINTS, 
	CODEPAGE=''ACP'', 
	DATAFILETYPE = ''char'',
	FIELDTERMINATOR='','', 
	ROWTERMINATOR = ''\n'', 
	KEEPIDENTITY, 
	TABLOCK
	); 
	');

  --gymnast
  EXECUTE ( N'BULK INSERT GYMNAST FROM ''' + @data_path + N'GYMNAST.csv''
  WITH (
	CHECK_CONSTRAINTS, 
	CODEPAGE=''ACP'', 
	DATAFILETYPE = ''char'',
	FIELDTERMINATOR='','', 
	ROWTERMINATOR = ''\n'', 
	KEEPIDENTITY, 
	TABLOCK
	); 
	');

  --prev experience
  EXECUTE ( N'BULK INSERT PREV_EXPERIENCE FROM ''' + @data_path + N'PREV_EXPERIENCE.csv''
  WITH (
	CHECK_CONSTRAINTS, 
	CODEPAGE=''ACP'', 
	DATAFILETYPE = ''char'',
	FIELDTERMINATOR='','', 
	ROWTERMINATOR = ''\n'', 
	KEEPIDENTITY, 
	TABLOCK
	); 
	');

  --score
  EXECUTE ( N'BULK INSERT SCORE FROM ''' + @data_path + N'SCORE.csv''
  WITH (
	CHECK_CONSTRAINTS, 
	CODEPAGE=''ACP'', 
	DATAFILETYPE = ''char'',
	FIELDTERMINATOR='','', 
	ROWTERMINATOR = ''\n'', 
	KEEPIDENTITY, 
	TABLOCK
	); 
	');

  --gymnast level
  EXECUTE ( N'BULK INSERT GYMNAST_LEVEL FROM ''' + @data_path + N'GYMNAST_LEVEL.csv''
  WITH (
	CHECK_CONSTRAINTS, 
	CODEPAGE=''ACP'', 
	DATAFILETYPE = ''char'',
	FIELDTERMINATOR='','', 
	ROWTERMINATOR = ''\n'', 
	KEEPIDENTITY, 
	TABLOCK
	); 
	');

  --level coach 
  EXECUTE ( N'BULK INSERT LEVEL_COACH FROM ''' + @data_path + N'LEVEL_COACH.csv''
  WITH (
	CHECK_CONSTRAINTS, 
	CODEPAGE=''ACP'', 
	DATAFILETYPE = ''char'',
	FIELDTERMINATOR='','', 
	ROWTERMINATOR = ''\n'', 
	KEEPIDENTITY, 
	TABLOCK
	); 
	');

	--list tables and row count as confirmation 
	GO
	SET NOCOUNT ON 
	SELECT 'LEVEL' AS "Table", COUNT(*) AS "Rows" FROM LEVEL UNION 
	SELECT 'EVENT', COUNT(*) FROM EVENT UNION 
	SELECT 'COACH', COUNT(*) FROM COACH UNION 
	SELECT 'HOUSEHOLD', COUNT(*) FROM HOUSEHOLD UNION 
	SELECT 'GYMNAST', COUNT(*) FROM GYMNAST UNION 
	SELECT 'PREV_EXPERIENCE', COUNT(*) FROM PREV_EXPERIENCE UNION 
	SELECT 'SCORE', COUNT(*) FROM SCORE UNION 
	SELECT 'GYMNAST_LEVEL', COUNT(*) FROM GYMNAST_LEVEL UNION 
	SELECT 'LEVEL_COACH', COUNT(*) FROM LEVEL_COACH
	ORDER BY 1; 
	SET NOCOUNT OFF
	GO 