--Written by: Tara Graeve
--Peer review by: Maddie Thompson
--Tara's Twisters Gymnastics: INFO 3240 Project Phase 1
--4/22/2020

USE TarasTwistersGymnastics
GO

--This function takes in a 2 letter string representing the event and returns a table containing the 
--average score for each gymnast on that event. This will be used to display all the averages on each 
--event through 1 query.

--check if function is already in db; drop it if it is and recreate it
IF EXISTS (
	SELECT * FROM sys.objects
	WHERE NAME = N'fn_EventAvg')
	DROP FUNCTION fn_EventAvg; 
GO 

CREATE FUNCTION fn_EventAvg(@InputEvent NCHAR(2))
RETURNS TABLE
AS
	RETURN (SELECT (Gymnast_FName + ' ' + Gymnast_LName) AS GymnastName, AVG(Score) AS "Average"
			FROM SCORE 
			INNER JOIN GYMNAST 
				ON SCORE.Gymnast_ID = GYMNAST.Gymnast_ID 
			WHERE Event_ID = @InputEvent
			GROUP BY Gymnast_FName, Gymnast_LName);   
GO 

--Test event average function 
SELECT  VT.GymnastName AS "Gymnast Name", VT.Average AS "VT Avg", UB.Average AS "UB Avg", 
BB.Average AS "BB Avg", FX.Average AS "FX Avg"
FROM dbo.fn_EventAvg('VT') AS VT
INNER JOIN dbo.fn_EventAvg('UB') AS UB
	ON VT.GymnastName = UB.GymnastName
INNER JOIN dbo.fn_EventAvg('BB') AS BB
	ON UB.GymnastName = BB.GymnastName
INNER JOIN dbo.fn_EventAvg('FX') AS FX
	ON BB.GymnastName = FX.GymnastName; 

GO 

--View: 
--Show averages and levels for each gymnast on each event in a format akin to what 
--a gymnast/someone familiar with gymnastics would expect: the VT - UB - BB - FX "headings"
--at the top are typical display format for gymnastics scores. 

--check if view is already in db; drop it if it is and recreate it
IF EXISTS (
	SELECT * FROM sys.objects
	WHERE NAME = N'GymnastAverages')
	DROP VIEW GymnastAverages; 
GO 

CREATE VIEW GymnastAverages
AS
	SELECT  VT.GymnastName, GYMNAST_LEVEL.Gymnast_Level, VT.Average AS "VT Avg", UB.Average AS "UB Avg", 
	BB.Average AS "BB Avg", FX.Average AS "FX Avg"
	FROM dbo.fn_EventAvg('VT') AS VT
	INNER JOIN dbo.fn_EventAvg('UB') AS UB
		ON VT.GymnastName = UB.GymnastName
	INNER JOIN dbo.fn_EventAvg('BB') AS BB
		ON UB.GymnastName = BB.GymnastName
	INNER JOIN dbo.fn_EventAvg('FX') AS FX
		ON BB.GymnastName = FX.GymnastName
	INNER JOIN GYMNAST 
		ON (GYMNAST.Gymnast_FName + ' ' + GYMNAST.Gymnast_LName) = FX.GymnastName
	INNER JOIN GYMNAST_LEVEL 
		ON GYMNAST.Gymnast_ID = GYMNAST_LEVEL.Gymnast_ID
WITH CHECK OPTION; 
GO 

--use view 
SELECT GymnastName AS "Gymnast Name", Gymnast_Level AS Level, 
		CONVERT(DECIMAL(5,3),"VT Avg") AS "VT Avg", 
		CONVERT(DECIMAL(5,3),"UB Avg") AS "UB Avg", 
		CONVERT(DECIMAL(5,3),"BB Avg") AS "BB Avg", 
		CONVERT(DECIMAL(5,3),"FX Avg") AS "FX Avg"
FROM GymnastAverages
ORDER BY 2 DESC,1; 

GO 

--check if function is already in db; drop it if it is and recreate it
IF EXISTS (
	SELECT * FROM sys.objects
	WHERE NAME = N'fn_GymnastID')
	DROP FUNCTION fn_GymnastID; 
GO 

--function takes in gymnast name and returns their ID
CREATE FUNCTION fn_GymnastID(@InputFName NVARCHAR(25), @InputLName NVARCHAR(35))
RETURNS INT 
BEGIN 
	DECLARE @Count INT, @ID INT 

	--check to see if gymnast specified is in database
	SELECT @Count = COUNT(*) 
	FROM GYMNAST
	WHERE Gymnast_FName = @InputFName AND Gymnast_LName = @InputLName; 

	--then set and return accordingly 
	BEGIN 
	IF @Count < 1 
		SET @ID = -1
	ELSE 
		SELECT @ID = Gymnast_ID 
		FROM GYMNAST 
		WHERE Gymnast_FName = @InputFName AND Gymnast_LName = @InputLName 
		RETURN @ID; 
	END; 
END; 

GO

--test function Gymnast ID 
--works
SELECT Gymnast_FName, Gymnast_LName 
FROM GYMNAST
WHERE Gymnast_ID = dbo.fn_GymnastID('Sierra','Brooks');

GO

--won't return anything -- not in database
SELECT Gymnast_FName, Gymnast_LName 
FROM GYMNAST
WHERE Gymnast_ID = dbo.fn_GymnastID('Sierra','Smith');

GO
--SPROC
--This procedure takes in a name and event and displays that gymnast's score history on the event

--check if procedure is already in db; drop it if it is and recreate it
IF EXISTS (
	SELECT * FROM sys.objects
	WHERE NAME = N'sp_ScoreHistory')
	DROP PROCEDURE sp_ScoreHistory; 
GO 

CREATE PROCEDURE sp_ScoreHistory(@InputFName NVARCHAR(25)=NULL,@InputLName NVARCHAR(35)=NULL,@InputEvent NCHAR(2)=NULL) 
AS 
BEGIN 
	BEGIN 
		--verify parameters
		IF @InputFName IS NULL OR @InputLName IS NULL OR @InputEvent IS NULL 
			PRINT 'Input parameter omitted: Correct format is: EXEC sp_ScoreHistory "<First Name>" "<Last Name>" "<Event>"'
	END; 
END; 
	--check that inputted gymnast is in the database and has a score 
BEGIN
	DECLARE @Count INT 
	SELECT @Count = COUNT(*) 
	FROM SCORE
	WHERE Gymnast_ID = dbo.fn_GymnastID(@InputFName,@InputLName) AND Event_ID = @InputEvent; 
	
	BEGIN
	--count > 0 means gymnast is in database
	IF @Count > 0
		SELECT CONVERT(NVARCHAR,Score_Date,1) Event_ID, Score
		FROM SCORE 
		WHERE Gymnast_ID = dbo.fn_GymnastID(@InputFName,@InputLName) AND Event_ID = @InputEvent
		ORDER BY Score_Date; 
	ELSE 
		BEGIN
			PRINT 'Gymnast and/or event not found'	
			RETURN(1); 
		END;
	END;
END; 
GO 

--test sproc 
--works
EXEC sp_ScoreHistory 'Sierra','Brooks','VT'
GO 
--won't work 
EXEC sp_ScoreHistory 'Sierra','Brooks','AA'
EXEC sp_ScoreHistory 'Sierra'

GO 
